/*
 * aderivation.cpp
 *
 *  Created on: 19. 1. 2014
 *      Author: Tomas Pecka
 */

#include <tclap/CmdLine.h>

#include <factory/XmlDataFactory.hpp>
#include <exception/AlibException.h>

#include "regexp/transform/RegExpDerivation.h"

int main(int argc, char** argv) {
	try {
		TCLAP::CmdLine cmd("Regexp derivative compute binary", ' ', "0.01");
		TCLAP::ValueArg<std::string> string(	"s",	"string",	"String to derivate by",	false,	"-",		"file");
		cmd.add( string );

		TCLAP::ValueArg<std::string> regexp(	"r",	"regexp",	"Regexp to derivate",		false,	"-",		"file");
		cmd.add( regexp );

		cmd.parse(argc, argv);

		std::deque<sax::Token> stringTokens;
		if(string.isSet()) {
			if(string.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(stringTokens);
			} else {
				sax::SaxParseInterface::parseFile(string.getValue(), stringTokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(stringTokens);
		}

		std::deque<sax::Token> regexpTokens;
		if(regexp.isSet()) {
			if(regexp.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(regexpTokens);
			} else {
				sax::SaxParseInterface::parseFile(regexp.getValue(), regexpTokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(regexpTokens);
		}

		alib::XmlDataFactory::toStdout(regexp::RegExpDerivation::derivation(alib::XmlDataFactory::fromTokens<regexp::RegExp>(regexpTokens), alib::XmlDataFactory::fromTokens<string::LinearString>(stringTokens)));
		return 0;
	} catch(const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
